@@ -27,27 +27,39 @@
 #include	<assert.h>
 #include	"emil.h"
 
-#ifdef	HAVE_FNMATCH_H
+#ifndef	NO_FNMATCH
 #include	<fnmatch.h>
+#ifndef	FNM_NOMATCH
+#define	FNM_NOMATCH	1	/* This MAY work!  941223/TW */
+#endif
 #else
-static	int	ematch(char * pattern, char * string)
+static	int	ematch(char * p, char * s)
 {
 	/* Returns 0 if match, 1 else. '*' is a wildcard */
-
-	for (;*pattern;pattern++)
-	if (*pattern == '*') {
-		while (*pattern=='*') pattern++;
-		if (*pattern==0) return 0;
-		while (*string) {
-			if (ematch(pattern,string)==0) return 0;
-			string++;
-		}
-		return	1;
-	}
-	else {
-		if (*pattern != *(string++)) return 1;
-	}
-	return	* string;
+	/* Suggest that you install fnmatch instead...   */
+#ifdef DEBUG
+  if (edebug)
+    fprintf(stderr, "-   ematch (no fnmatch).\n");
+#endif
+  for (;*p;p++) 
+    if (*p == '*') 
+      {
+	p++; 
+	if (*p=='\0') 
+	  return 0;
+	while (*s != '\0') 
+	  {
+	    if (ematch(p, s) == 0) 
+	      return 0; 
+	    else 
+	      s++;
+	  }
+	return	1;
+      } 
+    else 
+      if (*p != *(s++)) 
+	return 1;
+  return *s;
 }
 #endif
 
@@ -140,10 +152,19 @@ static	int	Ycmp(char * pattern, char * string)
 
 	/* Compare */
 
-#ifdef	HAVE_FNMATCH_H
-	cc = (fnmatch(pattern,string,0)==0);
+#ifndef	NO_FNMATCH
+	cc = fnmatch(pattern,string,0) != FNM_NOMATCH; /* 941223/TW */
 #else
-	cc = ematch(pattern,string);
+	cc = ematch(pattern,string) == 0;
+#endif
+#ifdef DEBUG
+	if (edebug)
+	  {
+	    if (cc)
+	      fprintf(stderr, "+   Ycmp: %s matches %s.\n", string, pattern);
+	    else
+	      fprintf(stderr, "+   Ycmp: %s does not match %s.\n", string, pattern);
+	  }
 #endif
 	return	cc;
 }
@@ -254,8 +275,8 @@ static const short yyrhs[] = {    -1,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-   195,   196,   199,   207,   209,   212,   214,   235,   236,   239,
-   242,   245,   248,   251,   256,   257,   261
+   216,   217,   220,   228,   230,   233,   235,   256,   257,   260,
+   263,   266,   269,   272,   277,   278,   282
 };
 
 static const char * const yytname[] = {   "$","error","$illegal.","COLON","COMMA",
